#!/usr/bin/env python3

"""
a demo for logging communcations between rlrwap and filter

Usage:
rlwrap -z './logger.py -l logger.log' telnet
"""

import os
import sys

if 'RLWRAP_FILTERDIR' in os.environ:
    sys.path.append(os.environ['RLWRAP_FILTERDIR'])
else:
    sys.path.append('.')

import rlwrapfilter
import argparse
import re

filter = rlwrapfilter.RlwrapFilter()

parser = argparse.ArgumentParser()

parser.add_argument('--logfile', '-l', nargs='?',
                    type=argparse.FileType('a'),
                    default=open("/tmp/filterlog." + str(os.getpid()), mode='a'))
args = parser.parse_args()
fd = args.logfile

# a message_handler is seldom used (as it cannot change messages, only examine them) 
# It gets called with the tag as its second argument
def logit(message, tag):
    tagname = filter.tag2name(tag)
    tagname = re.sub(r'^TAG_', '', tagname)
    mangled = message
    mangled = re.sub(r'\n', r'\\n', mangled)
    mangled = re.sub(r'\r', r'\\r', mangled)
    mangled = re.sub(r'\t', r'\\t', mangled)
    mangled = re.sub(r'\x1b', r'\\e', mangled)

    if (args.logfile):
        fd.write("{0} {1}\n".format(tagname, mangled))
    else:
        fd.write("{0:10s} {1}\n".format(tagname, mangled))

    fd.flush()
    return message


filter = rlwrapfilter.RlwrapFilter(message_handler=logit);


filter.help_text = "Usage: rlwrap -z 'logger [-l] logfile' <command>\n"\
                   + "log messages to a file (for debugging)\n"\
                   + "give logfile name as an argument, -l for long format\n"\
                   + "useful in a pipeline "\
                   + "(rlwrap -z 'pipeline logger in:filter:logger out')"


filter.run()



#!/usr/bin/env python3

"""a simple filter to tame commands that don't terminate their output with a newline"""

import sys
import os

if 'RLWRAP_FILTERDIR' in os.environ:
    sys.path.append(os.environ['RLWRAP_FILTERDIR'])
else:
    sys.path.append('.')

import rlwrapfilter

filter = rlwrapfilter.RlwrapFilter()
filter.help_text = ("Usage: rlwrap [-options] -z \"nl_and_then_prompt.py ['<prompt>']\" <command>\n" +
                   "terminate <command> output with a newline and display new <prompt>")


substitute_prompt = sys.argv[1] if len(sys.argv) == 2 else " > "


def handle_prompt(last_line):
    filter.send_output_oob(f"{last_line}\n")
    return substitute_prompt
    

filter.prompt_handler = handle_prompt
filter.run()




#!/usr/bin/env python3

# this is maybe the most practical of the filter examples. Is is also a test for rlwraps signal handling.
# At present, a CTRL-C in a pager will also kill rlwrap (bad)

import sys
import os

if 'RLWRAP_FILTERDIR' in os.environ:
    sys.path.append(os.environ['RLWRAP_FILTERDIR'])
else:
    sys.path.append('.')

import rlwrapfilter
import re

filter = rlwrapfilter.RlwrapFilter()
me = filter.name

raw_input = ''

filter.help_text = '\n'.join([
    "Usage: rlwrap -z {0} <command>".format(me),
    "Allow piping of <command>'s interactive output through pagers or other shell commands.",
    "When input of the form \"| shell pipeline\" is seen, <command>'s following output is sent through the pipeline\n"
    ])

pipeline = ''
prompt = ''
out_chunkno = 0
wait_text = "wait ..."


def input(message):
    global raw_input
    global pipeline
    input = None
    raw_input = message
    (input, bar, pipeline) = re.match(r'([^\|]*)(\|(.*))?', message).groups()
    return input


def output(message):
    global out_chunkno
    if (pipeline):
        if (out_chunkno == 0):
            out_chunkno += 1
            return wait_text
        else:
            return ""
    else:
        return message # replace first chunk by $wait_text


def prompt_handler(message):
    global pipeline
    prompt = message
    out_chunkno = 0
    if (pipeline):
        filter.send_output_oob("\x08" * len(wait_text) + "\n") # erase $wait_text and go to new line
        pipe = os.popen(pipeline, 'w')
        pipe.write(filter.cumulative_output)
        pipe.close()
        pipeline = None
        filter.send_output_oob("\n") # start prompt on new line
    return prompt


filter.prompts_are_never_empty = True
filter.input_handler = input
filter.output_handler = output
filter.prompt_handler = prompt_handler
filter.echo_handler = (lambda x: raw_input)

filter.run()
